/*
 ******************************************************************************
 *                                                                            *
 * CHDLCAPI.H - the 'C' header file for the Sangoma S508/S514 CHDLC code API. *
 *                                                                            *
 ******************************************************************************
*/

#ifndef _CHDLCAPI_
#define _CHDLCAPI_

/* ----------------------------------------------------------------------------
 *        Constants defining the shared memory control block (mailbox)
 * --------------------------------------------------------------------------*/

#define PRI_BASE_ADDR_MB_STRUCT 		 	0xE000 	/* the base address of the mailbox structure (primary port) */
#define SEC_BASE_ADDR_MB_STRUCT 		 	0xE800 	/* the base address of the mailbox structure (secondary port) */
#define NUMBER_MB_RESERVED_BYTES			0x0B		/* the number of reserved bytes in the mailbox header area */
#define SIZEOF_MB_DATA_BFR					2032		/* the size of the actual mailbox data area */

#pragma pack(1)

/* the control block mailbox structure */
typedef struct {
	unsigned char opp_flag;								/* the opp flag */
	unsigned char command;								/* the user command */
	unsigned short buffer_length;						/* the data length */
  	unsigned char return_code;							/* the return code */
	char MB_reserved[NUMBER_MB_RESERVED_BYTES];	/* reserved for later use */
	char data[SIZEOF_MB_DATA_BFR];					/* the data area */
} CHDLC_MAILBOX_STRUCT;



/* ----------------------------------------------------------------------------
 *                        Interface commands
 * --------------------------------------------------------------------------*/

/* global interface commands */
#define READ_GLOBAL_EXCEPTION_CONDITION	0x01	/* read a global exception condition from the adapter */
#define SET_GLOBAL_CONFIGURATION				0x02	/* set the global operational configuration */
#define READ_GLOBAL_CONFIGURATION			0x03	/* read the global configuration */
#define READ_GLOBAL_STATISTICS				0x04	/* retrieve the global statistics */
#define FLUSH_GLOBAL_STATISTICS				0x05	/* flush the global statistics */
#define SET_MODEM_STATUS						0x06	/* set the status of DTR and/or RTS */
#define READ_MODEM_STATUS						0x07	/* read the current status of CTS and DCD */
#define READ_COMMS_ERROR_STATS				0x08	/* read the communication error statistics */
#define FLUSH_COMMS_ERROR_STATS				0x09	/* flush the communication error statistics */
#define SET_TRACE_CONFIGURATION				0x0A	/* set the line trace configuration */
#define READ_TRACE_CONFIGURATION				0x0B	/* read the line trace configuration */
#define READ_TRACE_STATISTICS					0x0C	/* read the trace statistics */
#define FLUSH_TRACE_STATISTICS				0x0D	/* flush the trace statistics */
#define SET_FT1_CONFIGURATION					0x18	/* set the FT1 configuration */
#define READ_FT1_CONFIGURATION				0x19	/* read the FT1 configuration */
#define TRANSMIT_ASYNC_DATA_TO_FT1			0x1A	/* output asynchronous data to the FT1 */
#define RECEIVE_ASYNC_DATA_FROM_FT1			0x1B	/* receive asynchronous data from the FT1 */
#define FT1_MONITOR_STATUS_CTRL				0x1C	/* set the status of the FT1 monitoring */
#define READ_FT1_OPERATIONAL_STATS			0x1D	/* read the FT1 operational statistics */
#define SET_FT1_MODE								0x1E	/* set the operational mode of the FT1 module */

/* CHDLC-level interface commands */
#define READ_CHDLC_CODE_VERSION				0x20	/* read the CHDLC code version */
#define READ_CHDLC_EXCEPTION_CONDITION		0x21	/* read an CHDLC exception condition from the adapter */
#define SET_CHDLC_CONFIGURATION				0x22	/* set the CHDLC operational configuration */
#define READ_CHDLC_CONFIGURATION				0x23	/* read the current CHDLC operational configuration */
#define ENABLE_CHDLC_COMMUNICATIONS			0x24	/* enable CHDLC communications */
#define DISABLE_CHDLC_COMMUNICATIONS		0x25	/* disable CHDLC communications */
#define READ_CHDLC_LINK_STATUS				0x26	/* read the CHDLC link status */
#define READ_CHDLC_OPERATIONAL_STATS		0x27	/* retrieve the CHDLC operational statistics */
#define FLUSH_CHDLC_OPERATIONAL_STATS		0x28	/* flush the CHDLC operational statistics */
#define SET_CHDLC_INTERRUPT_TRIGGERS		0x30	/* set the CHDLC application interrupt triggers */
#define READ_CHDLC_INTERRUPT_TRIGGERS		0x31	/* read the CHDLC application interrupt trigger configuration */



/* ----------------------------------------------------------------------------
 *                     Return codes from interface commands
 * --------------------------------------------------------------------------*/

#define OK											0x00	/* the interface command was successfull */

/* return codes from global interface commands */
#define NO_GLOBAL_EXCEP_COND_TO_REPORT		0x01	/* there is no CHDLC exception condition to report */
#define LGTH_GLOBAL_CFG_DATA_INVALID		0x01	/* the length of the passed global configuration data is invalid */
#define LGTH_TRACE_CFG_DATA_INVALID			0x01	/* the length of the passed trace configuration data is invalid */
#define IRQ_TIMEOUT_VALUE_INVALID			0x02	/* an invalid application IRQ timeout value was selected */
#define TRACE_CONFIG_INVALID					0x02	/* the passed line trace configuration is invalid */
#define ADAPTER_OPERATING_FREQ_INVALID		0x03	/* an invalid adapter operating frequency was selected */
#define TRC_DEAC_TMR_INVALID					0x03	/* the trace deactivation timer is invalid */
#define FT1_ADPTR_NOT_PRESENT					0x06	/* the FT1 adapter is not present */
#define LGTH_FT1_CFG_DATA_INVALID 			0x07	/* the length of the passed FT1 configuration data is invalid */
#define LENGTH_FT1_TX_DATA_INVALID			0x07	/* the length of the FT1 asynchronous transmit data is invalid */
#define NO_FT1_RX_DATA_AVAILABLE				0x07	/* no FT1 asynchronous receive data is available */
#define LENGTH_FT1_MONITOR_DATA_INVALID	0x07	/* the length of the FT1 monitor data field is invalid */
#define FT1_OP_STATS_NOT_ENABLED				0x07	/* the FT1 operational statistics have not been enabled */
#define BAUD_RATE_INVALID_FOR_ENCODING		0x08	/* the selected FT1 baud rate is invalid for the encoding mode */
#define AUTO_FT1_CONFIG_NOT_COMPLETE		0x08	/* automatic configuring of the FT1 is not yet complete */
#define FT1_MODE_SELECTION_BUSY				0x08	/* the FT1 adapter is busy selecting the operational mode */
#define FT1_OP_STATS_NOT_AVAILABLE			0x08	/* the FT1 operational statistics are not currently available */
#define INVALID_FT1_CFG_DATA					0x09	/* the passed FT1 configuration data is invalid */
#define FT1_CONFIG_NOT_READ					0x09	/* the FT1 configuration was not read */
#define INVALID_FT1_STATUS_SELECTION		0x09	/* the FT1 status selection is invalid */
#define CHANNEL_BASE_INVALID					0x0A	/* the FT1 channel base is invalid for the configured baud rate */
#define FT1_TX_BUFFER_OCCUPIED				0x0B	/* the FT1 asynchronous transmit buffer is occupied */
#define AUTO_FT1_CFG_BUSY						0x0C	/* we are busy with an automatic FT1 configuration */
#define AUTO_FT1_CFG_FAIL_OP_MODE			0x0C	/* automatic FT1 configuration failed - FT1 not in operational mode */
#define AUTO_FT1_CFG_FAIL_INVALID_LINE		0x0D	/* automatic FT1 configuration failed - no valid line found */


/* return codes from command READ_GLOBAL_EXCEPTION_CONDITION */
#define EXCEP_MODEM_STATUS_CHANGE			0x10		/* a modem status change occurred */
#define EXCEP_TRC_DISABLED						0x11		/* the trace has been disabled */
#define EXCEP_APP_IRQ_TIMEOUT					0x12		/* an application IRQ timeout has occurred */

/* return codes from CHDLC-level interface commands */
#define NO_CHDLC_EXCEP_COND_TO_REPORT		0x21	/* there is no CHDLC exception condition to report */
#define CHDLC_COMMS_DISABLED					0x21	/* communications are not currently enabled */
#define CHDLC_COMMS_ENABLED					0x21	/* communications are currently enabled */
#define DISABLE_CHDLC_COMMS_BEFORE_CFG		0x21	/* CHDLC communications must be disabled before setting the configuration */
#define ENABLE_CHDLC_COMMS_BEFORE_CONN		0x21	/* communications must be enabled before using the CHDLC_CONNECT conmmand */
#define CHDLC_CFG_BEFORE_COMMS_ENABLED		0x22	/* perform a SET_CHDLC_CONFIGURATION before enabling comms */
#define LGTH_CHDLC_CFG_DATA_INVALID 		0x22	/* the length of the passed CHDLC configuration data is invalid */
#define LGTH_INT_TRIGGERS_DATA_INVALID		0x22	/* the length of the passed interrupt trigger data is invalid */
#define INVALID_IRQ_SELECTED					0x23	/* an invalid IRQ was selected in the SET_CHDLC_INTERRUPT_TRIGGERS */
#define INVALID_CHDLC_CFG_DATA				0x23	/* the passed CHDLC configuration data is invalid */
#define IRQ_TMR_VALUE_INVALID					0x24	/* an invalid application IRQ timer value was selected */
#define LARGER_PERCENT_TX_BFR_REQUIRED		0x24	/* a larger Tx buffer percentage is required */
#define LARGER_PERCENT_RX_BFR_REQUIRED		0x25	/* a larger Rx buffer percentage is required */
#define S514_BOTH_PORTS_SAME_CLK_MODE		0x26	/* S514 - both ports must have the same clocking mode */
#define INVALID_CMND_HDLC_STREAM_MODE		0x4E	/* the CHDLC interface command is invalid for HDLC streaming mode */
#define INVALID_CHDLC_COMMAND					0x4F	/* the defined CHDLC interface command is invalid */

/* return codes from command READ_CHDLC_EXCEPTION_CONDITION */
#define EXCEP_LINK_ACTIVE						0x30	/* the CHDLC link has become active */
#define EXCEP_LINK_INACTIVE_MODEM			0x31	/* the CHDLC link has become inactive (modem status) */
#define EXCEP_LINK_INACTIVE_KPALV			0x32	/* the CHDLC link has become inactive (keepalive status) */
#define EXCEP_IP_ADDRESS_DISCOVERED			0x33	/* the IP address has been discovered */
#define EXCEP_LOOPBACK_CONDITION				0x34	/* a loopback condition has occurred */



/* ----------------------------------------------------------------------------
 * Constants for the SET_GLOBAL_CONFIGURATION/READ_GLOBAL_CONFIGURATION commands
 * --------------------------------------------------------------------------*/

/* the global configuration structure */
typedef struct {
	unsigned short adapter_config_options;		/* adapter configuration options */
	unsigned short app_IRQ_timeout;				/* application IRQ timeout */
	unsigned long adapter_operating_frequency;	/* adapter operating frequency */
	unsigned short frame_transmit_timeout;		/* ??? David added it */
} GLOBAL_CONFIGURATION_STRUCT;

/* settings for the 'adapter_config_options' */
#define ADPTR_CFG_S514							0x0001 /* S514 adapter */

/* settings for the 'app_IRQ_timeout' */
#define MAX_APP_IRQ_TIMEOUT_VALUE			5000	/* the maximum permitted IRQ timeout */



/* ----------------------------------------------------------------------------
 *             Constants for the READ_GLOBAL_STATISTICS command
 * --------------------------------------------------------------------------*/

/* the global statistics structure */
typedef struct {
	unsigned short app_IRQ_timeout_count;        /* application IRQ timeout count */
} GLOBAL_STATS_STRUCT;



/* ----------------------------------------------------------------------------
 *             Constants for the READ_COMMS_ERROR_STATS command
 * --------------------------------------------------------------------------*/

/* the communications error statistics structure */
typedef struct {
	unsigned short Rx_overrun_err_count; 			/* receiver overrun error count */
	unsigned short CRC_err_count;						/* receiver CRC error count */
	unsigned short Rx_abort_count; 					/* abort frames received count */
	unsigned short Rx_dis_pri_bfrs_full_count;	/* receiver disabled count */
	unsigned short comms_err_stat_reserved_1;		/* reserved for later use */
	unsigned short sec_Tx_abort_msd_Tx_int_count;/* secondary port - abort frames Tx count (missed Tx interrupt) */
	unsigned short missed_Tx_und_int_count;		/* primary port - missed Tx underrun interrupt count */
												/* secondary port - abort frames Tx count (missed Tx underrun) */
	unsigned short sec_Tx_abort_count;				/* secondary port - abort frames Tx count */
	unsigned short DCD_state_change_count;			/* DCD state change count */
	unsigned short CTS_state_change_count;			/* CTS state change count */
} COMMS_ERROR_STATS_STRUCT;



/* ----------------------------------------------------------------------------
 *                  Constants used for line tracing
 * --------------------------------------------------------------------------*/

/* the trace configuration structure (SET_TRACE_CONFIGURATION/READ_TRACE_CONFIGURATION commands) */
typedef struct {
	unsigned char trace_config;						/* trace configuration */
	unsigned short trace_deactivation_timer;		/* trace deactivation timer */
	unsigned long ptr_trace_stat_el_cfg_struct;	/* a pointer to the line trace element configuration structure */
} LINE_TRACE_CONFIG_STRUCT;

/* 'trace_config' bit settings */
#define TRACE_INACTIVE							0x00	/* trace is inactive */
#define TRACE_ACTIVE								0x01	/* trace is active */
#define TRACE_DELAY_MODE						0x04	/* operate the trace in the delay mode */
#define TRACE_DATA_FRAMES						0x08	/* trace Data frames */
#define TRACE_SLARP_FRAMES						0x10	/* trace SLARP frames */
#define TRACE_CDP_FRAMES						0x20	/* trace CDP frames */

/* permitter range for the 'trace_deactivation_timer' */
#define MIN_TRC_DEAC_TMR_VAL					0		/* the minimum trace deactivation timer value */
#define MAX_TRC_DEAC_TMR_VAL					8000	/* the maximum trace deactivation timer value */

/* the line trace status element configuration structure */
typedef struct {
	unsigned short number_trace_status_elements;	/* number of line trace elements */
	unsigned long base_addr_trace_status_elements;/* base address of the trace element list */
	unsigned long next_trace_element_to_use;		/* pointer to the next trace element to be used */
	unsigned long base_addr_trace_buffer;			/* base address of the trace data buffer */
	unsigned long end_addr_trace_buffer;			/* end address of the trace data buffer */
} TRACE_STATUS_EL_CFG_STRUCT;

/* the line trace status element structure */
typedef struct {
	unsigned char opp_flag;								/* opp flag */
	unsigned short trace_length;						/* trace length */
	unsigned char trace_type;							/* trace type */
	unsigned short trace_time_stamp;					/* time stamp */
	unsigned short trace_reserved_1;					/* reserved for later use */
	unsigned long trace_reserved_2;					/* reserved for later use */
	unsigned long ptr_data_bfr;						/* pointer to the trace data buffer */
} TRACE_STATUS_ELEMENT_STRUCT;

/* the line trace statistics structure */
typedef struct {
	unsigned long frames_traced_count;				/* number of frames traced */
	unsigned long trc_frms_not_recorded_count;	/* number of trace frames discarded */
	unsigned short trc_disabled_internally_count;/* number of times the trace was disabled internally */
} LINE_TRACE_STATS_STRUCT;



/* ----------------------------------------------------------------------------
 *   Constants for the SET_FT1_CONFIGURATION/READ_FT1_CONFIGURATION command
 * --------------------------------------------------------------------------*/

/* the FT1 configuration structure */
typedef struct {
	unsigned short framing_mode;
	unsigned short encoding_mode;
	unsigned short line_build_out;
	unsigned short channel_base;
	unsigned short baud_rate_kbps;					/* the baud rate (in kbps) */	
	unsigned short clock_mode;
} FT1_CONFIGURATION_STRUCT;

/* settings for the 'framing_mode' */
#define ESF_FRAMING								0x00	/* ESF framing */
#define D4_FRAMING								0x01	/* D4 framing */

/* settings for the 'encoding_mode' */
#define B8ZS_ENCODING							0x00	/* B8ZS encoding */
#define AMI_ENCODING								0x01	/* AMI encoding */

/* settings for the 'line_build_out' */
#define LN_BLD_CSU_0dB_DSX1_0_to_133		0x00	/* set build out to CSU (0db) or DSX-1 (0-133ft) */
#define LN_BLD_DSX1_133_to_266				0x01	/* set build out DSX-1 (133-266ft) */
#define LN_BLD_DSX1_266_to_399				0x02	/* set build out DSX-1 (266-399ft) */
#define LN_BLD_DSX1_399_to_533				0x03	/* set build out DSX-1 (399-533ft) */
#define LN_BLD_DSX1_533_to_655				0x04	/* set build out DSX-1 (533-655ft) */
#define LN_BLD_CSU_NEG_7dB						0x05	/* set build out to CSU (-7.5db) */
#define LN_BLD_CSU_NEG_15dB					0x06	/* set build out to CSU (-15db) */
#define LN_BLD_CSU_NEG_22dB					0x07	/* set build out to CSU (-22.5db) */

/* settings for the 'channel_base' */
#define MIN_CHANNEL_BASE_VALUE				1		/* the minimum permitted channel base value */
#define MAX_CHANNEL_BASE_VALUE				24		/* the maximum permitted channel base value */

/* settings for the 'baud_rate_kbps' */
#define MIN_BAUD_RATE_KBPS						0		/* the minimum permitted baud rate (kbps) */
#define MAX_BAUD_RATE_KBPS 					1536	/* the maximum permitted baud rate (kbps) */
#define BAUD_RATE_FT1_AUTO_CONFIG			0xFFFF /* the baud rate used to trigger an automatic FT1 configuration */

/* settings for the 'clock_mode' */
#define CLOCK_MODE_NORMAL						0x00	/* clock mode set to normal (slave) */
#define CLOCK_MODE_MASTER						0x01	/* clock mode set to master */



/* ----------------------------------------------------------------------------
 *               Constants for the FT1_MONITOR_STATUS_CTRL command
 * --------------------------------------------------------------------------*/

#define DISABLE_FT1_STATUS_STATISTICS		0x00	/* disable the FT1 status and statistics monitoring */
#define ENABLE_READ_FT1_STATUS				0x01	/* read the FT1 operational status */
#define ENABLE_READ_FT1_OP_STATS				0x02	/* read the FT1 operational statistics */
#define FLUSH_FT1_OP_STATS						0x04	/* flush the FT1 operational statistics */



/* ----------------------------------------------------------------------------
 *               Constants for the reading the FT1 LEDs 
 * --------------------------------------------------------------------------*/

/* the 'parallel_port_A_input' parameter in the FT1_INFORMATION_STRUCT structure */
#define PP_A_RT_NOT_RED							0x01	/* the RT LED is not red */
#define PP_A_RT_NOT_GREEN						0x02	/* the RT LED is not green */
#define PP_A_LL_NOT_RED							0x04	/* the LL LED is not red */
#define PP_A_DL_NOT_RED							0x08	/* the DL LED is not red */
#define PP_A_RT_LL_DL_NOT_LIT					(PP_A_RT_NOT_RED | PP_A_RT_NOT_GREEN | PP_A_LL_NOT_RED | PP_A_DL_NOT_RED)

/* the 'parallel_port_B_input' parameter in the FT1_INFORMATION_STRUCT structure */
#define PP_B_RxD_NOT_GREEN						0x01	/* the RxD LED is not green */
#define PP_B_TxD_NOT_GREEN						0x02	/* the TxD LED is not green */
#define PP_B_ERR_NOT_GREEN						0x04	/* the ERR LED is not green */
#define PP_B_ERR_NOT_RED						0x08	/* the ERR LED is not red */
#define PP_B_INS_NOT_RED						0x10	/* the INS LED is not red */
#define PP_B_INS_NOT_GREEN						0x20	/* the INS LED is not green */
#define PP_B_ST_NOT_GREEN						0x40	/* the ST LED is not green */
#define PP_B_ST_NOT_RED							0x80	/* the ST LED is not red */
#define PP_B_ST_NOT_LIT							(PP_B_ST_NOT_GREEN | PP_B_ST_NOT_RED)



/* ----------------------------------------------------------------------------
 * Constants for the SET_CHDLC_CONFIGURATION/READ_CHDLC_CONFIGURATION command
 * --------------------------------------------------------------------------*/

/* the CHDLC configuration structure */
typedef struct {
	unsigned long baud_rate;							/* the baud rate */	
	unsigned short line_config_options;				/* line configuration options */
	unsigned short modem_config_options;			/* modem configuration options */
	unsigned short modem_status_timer;				/* timer for monitoring modem status changes */
	unsigned short CHDLC_API_options;				/* CHDLC API options */
	unsigned short CHDLC_protocol_options;			/* CHDLC protocol options */
	unsigned short percent_data_buffer_for_Tx;	/* percentage data buffering used for Tx */
	unsigned short CHDLC_statistics_options;		/* CHDLC operational statistics options */
	unsigned short max_CHDLC_data_field_length;	/* the maximum length of the CHDLC Data field */
	unsigned short transmit_keepalive_timer;		/* the transmit keepalive timer */
	unsigned short receive_keepalive_timer;		/* the receive keepalive timer */
	unsigned short keepalive_error_tolerance;		/* the receive keepalive error tolerance */
	unsigned short SLARP_request_timer;				/* the SLARP request timer */
	unsigned long IP_address;							/* the IP address */
	unsigned long IP_netmask;							/* the IP netmask */
	unsigned long ptr_shared_mem_info_struct;		/* a pointer to the shared memory area information structure */
	unsigned long ptr_CHDLC_Tx_stat_el_cfg_struct;/* a pointer to the transmit status element configuration structure */
	unsigned long ptr_CHDLC_Rx_stat_el_cfg_struct;/* a pointer to the receive status element configuration structure */
} CHDLC_CONFIGURATION_STRUCT;

/* settings for the 'line_config_options' */
#define INTERFACE_LEVEL_V35					0x0000 /* V.35 interface level */
#define INTERFACE_LEVEL_RS232					0x0001 /* RS-232 interface level */

/* settings for the 'modem_config_options' */
#define DONT_RAISE_DTR_RTS_ON_EN_COMMS		0x0001 /* don't automatically raise DTR and RTS when performing an */
																 /* ENABLE_CHDLC_COMMUNICATIONS command */
#define DONT_REPORT_CHG_IN_MODEM_STAT 		0x0002 /* don't report changes in modem status to the application */

/* bit settings for the 'CHDLC_protocol_options' byte */
#define IGNORE_DCD_FOR_LINK_STAT				0x0001 /* ignore DCD in determining the CHDLC link status */
#define IGNORE_CTS_FOR_LINK_STAT				0x0002 /* ignore CTS in determining the CHDLC link status */
#define IGNORE_KPALV_FOR_LINK_STAT			0x0004 /* ignore keepalive frames in determining the CHDLC link status */
#define HDLC_STREAMING_MODE					0x8000 /* operate in the HDLC streaming mode */

/* settings for the 'CHDLC_statistics_options' */
#define CHDLC_TX_DATA_BYTE_COUNT_STAT		0x0001 /* record the number of Data bytes transmitted */
#define CHDLC_RX_DATA_BYTE_COUNT_STAT		0x0002 /* record the number of Data bytes received */
#define CHDLC_TX_THROUGHPUT_STAT				0x0004 /* compute the Data frame transmit throughput */
#define CHDLC_RX_THROUGHPUT_STAT				0x0008 /* compute the Data frame receive throughput */

/* permitted minimum and maximum values for setting the CHDLC configuration */
#define PRI_MAX_BAUD_RATE_S508				2666666 /* primary port - maximum baud rate (S508) */
#define SEC_MAX_BAUD_RATE_S508				258064  /* secondary port - maximum baud rate (S508) */
#define PRI_MAX_BAUD_RATE_S514				2750000 /* primary port - maximum baud rate (S514) */
#define SEC_MAX_BAUD_RATE_S514				515625  /* secondary port - maximum baud rate (S514) */
#define MIN_PERMITTED_MODEM_TIMER			0		  /* minimum modem status timer */
#define MAX_PERMITTED_MODEM_TIMER			5000	  /* maximum modem status timer */
#define MAX_PERMITTED_Tx_KPALV_TIMER		60000	  /* maximum transmit keepalive timer */
#define PRI_MAX_NO_DATA_BYTES_IN_FRAME		15354	  /* primary port - maximum length of the CHDLC data field */
#define SEC_MAX_NO_DATA_BYTES_IN_FRAME		2048	  /* secondary port - maximum length of the CHDLC data field */
#define MIN_PERMITTED_Tx_KPALV_TIMER		0		  /* minimum transmit keepalive timer */
#define MAX_PERMITTED_Tx_KPALV_TIMER		60000	  /* maximum transmit keepalive timer */
#define MIN_PERMITTED_Rx_KPALV_TIMER		10		  /* minimum receive keepalive timer */
#define MAX_PERMITTED_Rx_KPALV_TIMER		60000	  /* maximum receive keepalive timer */
#define MIN_PERMITTED_KPALV_ERR_TOL			1		  /* minimum keepalive error tolerance count */
#define MAX_PERMITTED_KPALV_ERR_TOL			20		  /* maximum keepalive error tolerance count */
#define MIN_PERMITTED_SLARP_REQ_TIMER		0		  /* minimum transmit SLARP Request timer */
#define MAX_PERMITTED_SLARP_REQ_TIMER		60000	  /* maximum transmit SLARP Request timer */



/* ----------------------------------------------------------------------------
 *             Constants for the READ_CHDLC_LINK_STATUS command
 * --------------------------------------------------------------------------*/

/* the CHDLC status structure */
typedef struct {
	unsigned char CHDLC_link_status;					/* CHDLC link status */
	unsigned char no_Data_frms_for_app;				/* number of Data frames available for the application */
	unsigned char receiver_status;					/* receiver status (enabled/disabled) */
	unsigned char CHDLC_Tx_Rx_state;					/* internal CHDLC state */
} CHDLC_LINK_STATUS_STRUCT;

/* settings for the 'CHDLC_link_status' variable */
#define CHDLC_LINK_INACTIVE					0x00	/* the CHDLC link is inactive */
#define CHDLC_LINK_ACTIVE						0x01	/* the CHDLC link is active */



/* ----------------------------------------------------------------------------
 *           Constants for the READ_CHDLC_OPERATIONAL_STATS command
 * --------------------------------------------------------------------------*/

/* the CHDLC operational statistics structure */
typedef struct {

	/* Data frame transmission statistics */
	unsigned long Data_frames_Tx_count;				/* number of frames transmitted */
	unsigned long Data_bytes_Tx_count; 				/* number of bytes transmitted */
	unsigned long Data_Tx_throughput;				/* transmit throughput */
	unsigned long no_ms_for_Data_Tx_thruput_comp;/* millisecond time used for the Tx throughput computation */
	unsigned long Tx_Data_discard_lgth_err_count;/* number of Data frames discarded (length error) */
	unsigned long reserved_Data_frm_Tx_stat1;		/* reserved for later use */
	unsigned long reserved_Data_frm_Tx_stat2;		/* reserved for later use */
	unsigned long reserved_Data_frm_Tx_stat3;		/* reserved for later use */

	/* Data frame reception statistics */
	unsigned long Data_frames_Rx_count;				/* number of frames received */
	unsigned long Data_bytes_Rx_count;				/* number of bytes received */
	unsigned long Data_Rx_throughput;				/* receive throughput */
	unsigned long no_ms_for_Data_Rx_thruput_comp;/* millisecond time used for the Rx throughput computation */
	unsigned long Rx_Data_discard_short_count;	/* received Data frames discarded (too short) */
	unsigned long Rx_Data_discard_long_count;		/* received Data frames discarded (too long) */
	unsigned long Rx_Data_discard_inactive_count;/* received Data frames discarded (link inactive) */
	unsigned long reserved_Data_frm_Rx_stat1;		/* reserved for later use */

	/* SLARP frame transmission/reception statistics */
	unsigned long CHDLC_SLARP_REQ_Tx_count;		/* number of SLARP Request frames transmitted */
	unsigned long CHDLC_SLARP_REQ_Rx_count;		/* number of SLARP Request frames received */
	unsigned long CHDLC_SLARP_REPLY_Tx_count;		/* number of SLARP Reply frames transmitted */
	unsigned long CHDLC_SLARP_REPLY_Rx_count;		/* number of SLARP Reply frames received */

	unsigned long CHDLC_SLARP_KPALV_Tx_count;		/* number of SLARP keepalive frames transmitted */
	unsigned long CHDLC_SLARP_KPALV_Rx_count;		/* number of SLARP keepalive frames received */

	unsigned long reserved_SLARP_stat1;				/* reserved for later use */
	unsigned long reserved_SLARP_stat2;				/* reserved for later use */

	/* CDP frame transmission/reception statistics */
	unsigned long CHDLC_CDP_Tx_count;				/* number of CDP frames transmitted */
	unsigned long CHDLC_CDP_Rx_count;				/* number of CDP frames received */
	unsigned long reserved_CDP_stat1;				/* reserved for later use */
	unsigned long reserved_CDP_stat2;				/* reserved for later use */
	unsigned long reserved_CDP_stat3;				/* reserved for later use */
	unsigned long reserved_CDP_stat4;				/* reserved for later use */
	unsigned long reserved_CDP_stat5;				/* reserved for later use */
	unsigned long reserved_CDP_stat6;				/* reserved for later use */

	/* Incoming frames with a format error statistics */
	unsigned short Rx_frm_incomp_CHDLC_hdr_count;/* frames received of with incomplete Cisco HDLC header */
	unsigned short Rx_frms_too_long_count;			/* frames received of excessive length count */
	unsigned short Rx_invalid_CHDLC_addr_count;	/* frames received with an invalid CHDLC address count */
	unsigned short Rx_invalid_CHDLC_ctrl_count;	/* frames received with an invalid CHDLC control field count */
	unsigned short Rx_invalid_CHDLC_type_count;	/* frames received of an invalid CHDLC frame type count */
	unsigned short Rx_SLARP_invalid_code_count;	/* SLARP frame received with an invalid packet code */
	unsigned short Rx_SLARP_Reply_bad_IP_addr;	/* SLARP Reply received - bad IP address */
	unsigned short Rx_SLARP_Reply_bad_netmask;	/* SLARP Reply received - bad netmask */
	unsigned long reserved_frm_format_err1;		/* reserved for later use */
	unsigned long reserved_frm_format_err2;		/* reserved for later use */
	unsigned long reserved_frm_format_err3;		/* reserved for later use */
	unsigned long reserved_frm_format_err4;		/* reserved for later use */

	/* CHDLC timeout/retry statistics */
	unsigned short SLARP_Rx_keepalive_TO_count;	/* timeout count for incoming SLARP frames */
	unsigned short SLARP_Request_TO_count;			/* timeout count for SLARP Request frames */
	unsigned long To_retry_reserved_stat1;			/* reserved for later use */
	unsigned long To_retry_reserved_stat2;			/* reserved for later use */
	unsigned long To_retry_reserved_stat3;			/* reserved for later use */

	/* CHDLC link active/inactive and loopback statistics */
	unsigned short link_active_count;				/* number of times that the link went active */
	unsigned short link_inactive_modem_count;		/* number of times that the link went inactive (modem failure) */
	unsigned short link_inactive_keepalive_count;/* number of times that the link went inactive (keepalive failure) */
	unsigned short link_looped_count;				/* link looped count */
	unsigned long link_status_reserved_stat1;		/* reserved for later use */
	unsigned long link_status_reserved_stat2;		/* reserved for later use */

	/* miscellaneous statistics */
	unsigned long reserved_misc_stat1;				/* reserved for later use */
	unsigned long reserved_misc_stat2;				/* reserved for later use */
	unsigned long reserved_misc_stat3;				/* reserved for later use */
	unsigned long reserved_misc_stat4;				/* reserved for later use */

} CHDLC_OPERATIONAL_STATS_STRUCT;



/* ----------------------------------------------------------------------------
 *                 Constants for using application interrupts
 * --------------------------------------------------------------------------*/

/* the structure used for the SET_CHDLC_INTERRUPT_TRIGGERS/READ_CHDLC_INTERRUPT_TRIGGERS command */
typedef struct {
	unsigned char CHDLC_interrupt_triggers;		/* CHDLC interrupt trigger configuration */
	unsigned char IRQ;									/* IRQ to be used */
	unsigned short interrupt_timer;					/* interrupt timer */
	unsigned short misc_interrupt_bits;				/* miscellaneous interrupt bits */
} CHDLC_INT_TRIGGERS_STRUCT;

/* 'CHDLC_interrupt_triggers' bit settings */
#define APP_INT_ON_RX_FRAME					0x01	/* interrupt on Data frame reception */
#define APP_INT_ON_TX_FRAME					0x02	/* interrupt when an Data frame may be transmitted */
#define APP_INT_ON_COMMAND_COMPLETE			0x04	/* interrupt when an interface command is complete */
#define APP_INT_ON_TIMER					0x08	/* interrupt on a defined millisecond timeout */
#define APP_INT_ON_GLOBAL_EXCEP_COND 		0x10	/* interrupt on a global exception condition */
#define APP_INT_ON_CHDLC_EXCEP_COND			0x20	/* interrupt on an CHDLC exception condition */
#define APP_INT_ON_TRACE_DATA_AVAIL			0x80	/* interrupt when trace data is available */

/* 'interrupt_timer' limitation */
#define MAX_INTERRUPT_TIMER_VALUE 			60000		/* the maximum permitted timer interrupt value */

/* interrupt types indicated at 'interrupt_type' byte of the INTERRUPT_INFORMATION_STRUCT */
#define NO_APP_INTS_PEND						0x00	/* no interrups are pending */
#define RX_APP_INT_PEND							0x01	/* a receive interrupt is pending */
#define TX_APP_INT_PEND							0x02	/* a transmit interrupt is pending */
#define COMMAND_COMPLETE_APP_INT_PEND		0x04	/* a 'command complete' interrupt is pending */
#define TIMER_APP_INT_PEND						0x08	/* a timer interrupt is pending */
#define GLOBAL_EXCEP_COND_APP_INT_PEND 	0x10	/* a global exception condition interrupt is pending */
#define CHDLC_EXCEP_COND_APP_INT_PEND 		0x20	/* an CHDLC exception condition interrupt is pending */
#define TRACE_DATA_AVAIL_APP_INT_PEND		0x80	/* a trace data available interrupt is pending */

/* modem status changes */
#define DCD_HIGH			0x08
#define CTS_HIGH			0x20

/* ----------------------------------------------------------------------------
 *                   Constants for Data frame transmission
 * --------------------------------------------------------------------------*/

/* the Data frame transmit status element configuration structure */
typedef struct {
	unsigned short number_Tx_status_elements;		/* number of transmit status elements */
	unsigned long base_addr_Tx_status_elements;	/* base address of the transmit element list */
	unsigned long next_Tx_status_element_to_use;	/* pointer to the next transmit element to be used */
} CHDLC_TX_STATUS_EL_CFG_STRUCT;

/* the Data frame transmit status element structure */
typedef struct {
	unsigned char opp_flag;								/* opp flag */
	unsigned short frame_length;						/* length of the frame to be transmitted */
	unsigned char reserved_1;							/* reserved for internal use */
	unsigned long reserved_2;							/* reserved for internal use */
	unsigned long reserved_3;							/* reserved for internal use */
	unsigned long ptr_data_bfr;						/* pointer to the data area */
} CHDLC_DATA_TX_STATUS_EL_STRUCT;



/* ----------------------------------------------------------------------------
 *                   Constants for Data frame reception
 * --------------------------------------------------------------------------*/

/* the Data frame receive status element configuration structure */
typedef struct {
	unsigned short number_Rx_status_elements;		/* number of receive status elements */
	unsigned long base_addr_Rx_status_elements;	/* base address of the receive element list */
	unsigned long next_Rx_status_element_to_use;	/* pointer to the next receive element to be used */
	unsigned long base_addr_Rx_buffer;				/* base address of the receive data buffer */
	unsigned long end_addr_Rx_buffer;				/* end address of the receive data buffer */
} CHDLC_RX_STATUS_EL_CFG_STRUCT;

/* the Data frame receive status element structure */
typedef struct {
	unsigned char opp_flag;								/* opp flag */
	unsigned short frame_length;						/* length of the received frame */
	unsigned char error_flag;							/* frame errors (HDLC_STREAMING_MODE) */
	unsigned short time_stamp;							/* receive time stamp (HDLC_STREAMING_MODE) */
	unsigned long reserved_1;							/* reserved for internal use */
	unsigned short reserved_2;							/* reserved for internal use */
	unsigned long ptr_data_bfr;						/* pointer to the data area */
} CHDLC_DATA_RX_STATUS_EL_STRUCT;



/* ----------------------------------------------------------------------------
 *         Constants defining the shared memory information area
 * --------------------------------------------------------------------------*/

/* the global information structure */
typedef struct {
 	unsigned char global_status;						/* global status */
 	unsigned char modem_status;						/* current modem status */
 	unsigned char global_excep_conditions;			/* global exception conditions */
	unsigned char glob_info_reserved[5];			/* reserved */
	unsigned char code_name[4];						/* code name */
	unsigned char code_version[4];					/* code version */
} GLOBAL_INFORMATION_STRUCT;

/* the CHDLC information structure */
typedef struct {
	unsigned char CHDLC_status;						/* CHDLC status */
 	unsigned char CHDLC_excep_conditions;			/* CHDLC exception conditions */
	unsigned char CHDLC_info_reserved[14];			/* reserved */
} CHDLC_INFORMATION_STRUCT;

/* the interrupt information structure */
typedef struct {
 	unsigned char interrupt_type;						/* type of interrupt triggered */
 	unsigned char interrupt_permission;				/* interrupt permission mask */
	unsigned char int_info_reserved[14];			/* reserved */
} INTERRUPT_INFORMATION_STRUCT;

/* the FT1 information structure */
typedef struct {
 	unsigned char parallel_port_A_input;			/* input - parallel port A */
 	unsigned char parallel_port_B_input;			/* input - parallel port B */
	unsigned char FT1_info_reserved[14];			/* reserved */
} FT1_INFORMATION_STRUCT;

/* the shared memory area information structure */
typedef struct {
	GLOBAL_INFORMATION_STRUCT global_info_struct;		/* the global information structure */
	CHDLC_INFORMATION_STRUCT CHDLC_info_struct;			/* the CHDLC information structure */
	INTERRUPT_INFORMATION_STRUCT interrupt_info_struct;/* the interrupt information structure */
	FT1_INFORMATION_STRUCT FT1_info_struct;				/* the FT1 information structure */
} SHARED_MEMORY_INFO_STRUCT;

#pragma pack()

#endif //_CHDLCAPI_