// Sample program for Sangoma CHDLC streaming API.
// Copyright (c) 2001 Sangoma Technologies Inc.
//
// Target: Windows 2000/XP. Win32 Console.
//

//
// For Version 5,0,0,0 and later of the API driver (strmwdm.sys)
//
#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <conio.h>


#include "CHDLCAPI.H"
#include "sangstrm.h"

/*
#include <wanpipe\cpipe\chdlcapi.h>
#include <hdlc_strm_api\Sangstrm.H>
*/

CHDLC_MAILBOX_STRUCT * pcb;
UCHAR cbdata[SIZEOF_MB_DATA_BFR];

//buffer used for receiving data
UCHAR rx_data[sizeof(CHDLC_DATA_RX_STATUS_EL_STRUCT) + PRI_MAX_NO_DATA_BYTES_IN_FRAME];

//structure used for sending data
TX_DATA_STRUCT tx_data;

HANDLE hTxWaitHandle;

ULONG good_rx_frames_count = 0;
ULONG good_rx_bytes_count = 0;

//for counting frames with CRC/Abort errors
ULONG bad_rx_frames_count = 0;

ULONG tx_bytes_count = 0;
ULONG tx_frames_count = 0;

UCHAR tx_test_byte = 0;
UCHAR rx_test_byte = 0;

BOOLEAN stop_tx = FALSE;
int bad_rx_counter = 0;

char * device_name = "\\\\.\\SANGSTRM0";	//Windows 2000, S508 and S514

char c;

#define TX_LENGTH	1200

//
//Additional initialization should be done
//for S5144 and S5147 T1E1 cards. Not needed if Serial card is used.
//
//#define BUILD_FOR_T1E1_CARD

#if defined BUILD_FOR_T1E1_CARD
 //define only one of these:
 #define BUILD_T1
 //#define BUILD_E1
#endif

#if defined (BUILD_T1) && defined (BUILD_E1)
#error "Can not build for both T1 and E1!"
#endif

UCHAR 
DoCommand(
	HANDLE drv, 
	CHDLC_MAILBOX_STRUCT * pcb
	);

UCHAR 
DoWriteCommand(
	HANDLE drv, 
	TX_DATA_STRUCT * pTx
	);

USHORT 
DoReadCommand(
	HANDLE drv, 
	PUCHAR dat, 
	ULONG maxl
	);

VOID 
ConfigureTECardCommand(
	HANDLE drv, 
	TE_CFG_T * te_cfg
	);

VOID 
TE1MonitorCommand(
	HANDLE drv, 
	CHDLC_MAILBOX_STRUCT * mb
	);

DWORD 
RxThreadFunc(
	LPDWORD lpdwParam
	);

DWORD 
TxThreadFunc(
	LPDWORD lpdwParam
	);

int 
PrintRxData(
	PSTREAMING_READ_INFO pri
	);

void print_stats(HANDLE drv);

/////////////////////////////////////////

VOID 
ReadDeviceStatistics(
	HANDLE drv, 
	DEVICE_STATISTICS* p_dev_stats
	);
void get_device_stats(HANDLE drv);

VOID 
SetDeviceConfiguration(
	HANDLE drv, 
	DEVICE_CONFIGURATION* p_dev_cfg
	);

int set_device_configuration(HANDLE drv);

/////////////////////////////////////////

void main()
{
	HANDLE drv;
	DWORD ln, retry_count;
	CHDLC_CONFIGURATION_STRUCT * strc;

#ifdef BUILD_FOR_T1E1_CARD
	TE_CFG_T te_cfg;
#endif

    DWORD dwThreadId, dwThrdParam = 1; 
    HANDLE hRxThread, hTxThread; 

	pcb = (CHDLC_MAILBOX_STRUCT *)cbdata;

	// Open device for general commands
	drv = CreateFile(
			device_name, 
			GENERIC_READ | GENERIC_WRITE, 
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			(LPSECURITY_ATTRIBUTES)NULL, 
			OPEN_EXISTING,
			FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH,
			(HANDLE)NULL
			);

    if (drv == INVALID_HANDLE_VALUE)
	{	printf("Unable to open %s!\n", device_name);
		return;
	}

#ifdef BUILD_FOR_T1E1_CARD
		// clear the TE1 configuration structure
		memset(&te_cfg, 0x00, sizeof(te_cfg));

#if defined (BUILD_T1)

		//set T1 configuration	
		te_cfg.media	= WANOPT_MEDIA_T1;
		te_cfg.lcode	= WANOPT_LC_B8ZS;//WANOPT_LC_AMI;
		te_cfg.frame	= WANOPT_FR_ESF;
		te_cfg.lbo		= WANOPT_T1_LBO_0_DB;
		te_cfg.te_clock	= WANOPT_NORMAL_CLK;
		te_cfg.active_ch= TE_ENABLE_ALL_CHANNELS;

#elif defined (BUILD_E1)

		//set E1 configuration	
		te_cfg.media	= WANOPT_MEDIA_E1;
		te_cfg.lcode	= WANOPT_LC_HDB3;
		te_cfg.frame	= WANOPT_FR_NCRC4;
		te_cfg.lbo		= WANOPT_T1_LBO_0_DB;
		te_cfg.te_clock	= WANOPT_NORMAL_CLK;
		te_cfg.active_ch= TE_ENABLE_ALL_CHANNELS;
#endif

		printf("Setting %s configuration.\n", 
			(te_cfg.media == WANOPT_MEDIA_T1) ? "T1" : "E1");

		ConfigureTECardCommand(
			drv, 
			&te_cfg
			);

		printf("ConfigureTECardCommand return_code : ");

		switch(te_cfg.return_code)
		{
		case COMMAND_PERFORMED_SUCCESSFULLY:
			printf("COMMAND_PERFORMED_SUCCESSFULLY\n");
			break;

		case DO_DISABLE_CHDLC_COMMUNICATIONS:
			printf("DO_DISABLE_CHDLC_COMMUNICATIONS\n");
			break;

		case TE_CONFIGURATION_FAILED:
			printf("TE_CONFIGURATION_FAILED\n");
			break;
		}

		if(te_cfg.return_code != COMMAND_PERFORMED_SUCCESSFULLY)
		{	// close the open handle
			CloseHandle(drv);
			return;
		}
#endif //BUILD_FOR_T1E1_CARD

	// clear the configuration data area
	memset(cbdata, 0, sizeof(cbdata));

    // set up for a global configuration
	strc = (CHDLC_CONFIGURATION_STRUCT *)pcb->data;

	pcb->command = SET_CHDLC_CONFIGURATION;
	pcb->buffer_length = sizeof(CHDLC_CONFIGURATION_STRUCT);

#if !defined(BUILD_FOR_T1E1_CARD)	

	//(for TE1 cards this setting is discarded)
	printf("\nClock: 0. External, 1. 56 kbps, 2. 128 kbps, 3. 1600 kbps, 4. 2750 kbps");
	c = getche();

	switch(c)
	{
	case '0':
		strc->baud_rate = 0;
		break;
	case '1':
		strc->baud_rate = 56000;
		break;
	case '2':
		strc->baud_rate = 128000;
		break;
	case '3':
		strc->baud_rate = 1600000;
		break;
	case '4':
		strc->baud_rate = PRI_MAX_BAUD_RATE_S514;
		break;
	default:
		strc->baud_rate = 56000;
	}

	//(for TE1 cards this setting is discarded)
	printf("\nConfigure for 'r's-232 or 'v'.35? ");
	c = getche();
	if (c == 'r')
		strc->line_config_options = INTERFACE_LEVEL_RS232;
	else
		strc->line_config_options = INTERFACE_LEVEL_V35;
	printf("\n");

#endif //BUILD_FOR_T1E1_CARD

	strc->CHDLC_protocol_options = HDLC_STREAMING_MODE;	

	strc->percent_data_buffer_for_Tx = 50;

	strc->max_CHDLC_data_field_length = 1500;

	printf("Performing SET_CHDLC_CONFIGURATION\n");

	// perform the command
	DoCommand(drv, pcb);

	// if bad return_code don't continue, find what's wrong.
	printf("Rcode = 0x%02X, blength = %u\n", 
		pcb->return_code, pcb->buffer_length);

	for (ln=0; ln < (DWORD)pcb->buffer_length; ln++)
		printf("%02X ", pcb->data[ln]);
	printf("\n");

	if(pcb->return_code)
	{	// close the open handle
		CloseHandle(drv);
		return;
	}

	//Set device configuration - only if loss of frames is detected,
	//usually needed for short frames (~ 50 bytes long).
	//For most cases there is no need to change driver default.
	//Call get_device_stats() to find if any data was lost due
	//rx queue overflow.
	/*
	if(set_device_configuration(drv))
	{	CloseHandle(drv);
		return;
	}
	*/

	printf("\nPress any key to enable communications.");
	getch();

	// enable the transmitter and receiver (check the return_code!).
	pcb->command = ENABLE_CHDLC_COMMUNICATIONS;
	pcb->buffer_length = 0;
	printf("Performing ENABLE_CHDLC_COMMUNICATIONS\n");

	DoCommand(drv, pcb);

	// if bad return_code don't continue, find what's wrong.
	printf("Rcode = %02X, blength = %u\n", 
		pcb->return_code, pcb->buffer_length);

	if(pcb->return_code)
	{	// close the open handle
		CloseHandle(drv);
		return;
	}

	//start a thread for reading only
    hRxThread = CreateThread( 
        NULL,                       /* no security attributes        */ 
        0,                          /* use default stack size        */ 
        (LPTHREAD_START_ROUTINE) RxThreadFunc, /* thread function      */ 
        &dwThrdParam,               /* argument to thread function   */ 
        0,                          /* use default creation flags    */ 
        &dwThreadId					/* returns the thread identifier */ 
		);

	//create Tx wait object
	hTxWaitHandle = CreateEvent( NULL, FALSE, FALSE, NULL);
	if(hTxWaitHandle != NULL)
	{	//start a thread for writing only
		hTxThread = CreateThread( 
			NULL,                       /* no security attributes        */ 
			0,                          /* use default stack size        */ 
			(LPTHREAD_START_ROUTINE) TxThreadFunc, /* thread function      */ 
			&dwThrdParam,               /* argument to thread function   */ 
			0,                          /* use default creation flags    */ 
			&dwThreadId					/* returns the thread identifier */ 
			);
	}else
	{	printf("Failed to create Tx event!!\n");
	}

	// Loop around waiting for user to request the transmission of a frame
	// or to do some other command.
	do{
		printf("\n Press 'q' to quit or 'f' to get RX/TX statistics\n");
		printf("\t or 's' to send a frame.\n");

		c=getch();
		printf("\n");

		if(c == 'f')
		{	print_stats(drv);
			get_device_stats(drv);
		}
		else if(c == 's')
		{	if(hTxWaitHandle != NULL)
			{	//send a frame
				SetEvent(hTxWaitHandle);
			}
		}

	} while (c != 'q');

	//print program and card statistics
	print_stats(drv);
	//print API device statistics
	get_device_stats(drv);

	printf("\nPress any key to disable comms.\n");
	getch();

	//
	// Turn off the receiver/transmitter - this also cancels
	//any pending read/write commands.
	//
	retry_count = 0;
	do {
		pcb->command = DISABLE_CHDLC_COMMUNICATIONS;
		pcb->buffer_length = 0;
		DoCommand(drv, pcb);
		printf("Disable comms rcode = 0x%02X.\n", pcb->return_code);
	} while (pcb->return_code && retry_count++ < 10);

	printf("main(): Press any key to close device...\n");
	getch();

	// close the open handle
	CloseHandle(drv);
}

// perform a standard mailbox command
UCHAR 
DoCommand(
	HANDLE drv, 
	CHDLC_MAILBOX_STRUCT * pcb
	)
{
	DWORD ln;

	if (DeviceIoControl(
		drv,
		IoctlCodeCblockCommand,
		(LPVOID)pcb,
		(ULONG)pcb->buffer_length + 16L,
		(LPVOID)pcb,
		sizeof(cbdata),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		) == FALSE)
		pcb->return_code = 0x7d;		// STATUS_UNSUCCESSFUL
	return(pcb->return_code);
}

//
// To transmit a frame do blocking write command.
// On return check 'operation_status' for frame passed for transmission.
// 
UCHAR 
DoWriteCommand(
	HANDLE drv, 
	TX_DATA_STRUCT * pTx
	)
{
	DWORD ln;

	if(DeviceIoControl(
						drv,
						IoctlCodeWriteCommand,
						(LPVOID)pTx,
						(ULONG)sizeof(TX_DATA_STRUCT),
						(LPVOID)pTx,
						sizeof(TX_DATA_STRUCT),
						(LPDWORD)(&ln),
						(LPOVERLAPPED)NULL
						) == FALSE
	   )
		return (0x7d);		// STATUS_UNSUCCESSFUL
	else
		return (0);
}

// do blocking read command
// received frame is stored in user buffer on return
USHORT 
DoReadCommand(
	HANDLE drv, 
	PUCHAR dat, 
	ULONG maxl
	)
{
	DWORD ln;
	USHORT rc=0;

	if (DeviceIoControl(
		drv,
		IoctlCodeReadCommand,
		(LPVOID)NULL,
		0L,
		(LPVOID)dat,
		maxl,
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		) == FALSE)
		rc = 1;				// STATUS_UNSUCCESSFUL

	return rc;
}

VOID 
ConfigureTECardCommand(
	HANDLE drv, 
	TE_CFG_T * te_cfg
	)
{
	DWORD ln;

	DeviceIoControl(
		drv,
		IoctlSetTEConfiguration,
		(LPVOID)te_cfg,
		sizeof(TE_CFG_T),
		(LPVOID)te_cfg,
		sizeof(TE_CFG_T),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}

VOID 
TE1MonitorCommand(
	HANDLE drv, 
	CHDLC_MAILBOX_STRUCT * mb
	)
{
	DWORD ln;

	DeviceIoControl(
		drv,
		Ioctl_TE_56K_MonitorCommand,
		(LPVOID)mb,
		sizeof(CHDLC_MAILBOX_STRUCT),
		(LPVOID)mb,
		sizeof(CHDLC_MAILBOX_STRUCT),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}

VOID 
GetMemoryDump(
	HANDLE drv, 
	MEMORY_DUMP * pMem_dump
	)
{
	DWORD ln;

	DeviceIoControl(
		drv,
		Ioctl_GET_MEM_DUMP,
		(LPVOID)pMem_dump,
		sizeof(MEMORY_DUMP),
		(LPVOID)pMem_dump,
		sizeof(MEMORY_DUMP),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}

VOID 
ReadDeviceStatistics(
	HANDLE drv, 
	DEVICE_STATISTICS* p_dev_stats
	)
{
	DWORD ln;

	DeviceIoControl(
		drv,
		IoctlReadDeviceStatistics,
		(LPVOID)p_dev_stats,
		sizeof(DEVICE_STATISTICS),
		(LPVOID)p_dev_stats,
		sizeof(DEVICE_STATISTICS),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}

VOID 
SetDeviceConfiguration(
	HANDLE drv, 
	DEVICE_CONFIGURATION* p_dev_cfg
	)
{
	DWORD ln;

	DeviceIoControl(
		drv,
		IoctlSetDeviceConfiguration,
		(LPVOID)p_dev_cfg,
		sizeof(DEVICE_CONFIGURATION),
		(LPVOID)p_dev_cfg,
		sizeof(DEVICE_CONFIGURATION),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}

// Read Thread
DWORD 
RxThreadFunc(
	LPDWORD lpdwParam
	)
{ 
	HANDLE	drv_rx;
	PSTREAMING_READ_INFO pri;

    printf("\nRxThreadFunc:  thread parameter=%d\n", *lpdwParam); 

	//open another handle strictly for reading
	drv_rx = CreateFile(
			device_name, 
			GENERIC_READ | GENERIC_WRITE,
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			(LPSECURITY_ATTRIBUTES)NULL,
			OPEN_EXISTING,
			FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH,
			(HANDLE)NULL
			);

    if (drv_rx == INVALID_HANDLE_VALUE)
	{	printf("\nUnable to open %s for reading!\n", device_name);
		return(0);
	}

	while (DoReadCommand(drv_rx, rx_data, sizeof(rx_data)) == 0)
	{	printf("\nRead Thread:Waiting to read something...\n");

		//use our special buffer at rxdata to hold received frame
		pri = (PSTREAMING_READ_INFO)rx_data;

		if(pri->Status == RX_DATA_TIMEOUT)
		{	//no data in 2 seconds, try again.
			printf("Timeout on read.\n");
		}else if(pri->Status == RX_DATA_AVAILABLE)
		{	PrintRxData(pri);
		}
	}

	//If the read command returned but with no frame,
	//DISABLE_CHDLC_COMMUNICATIONS was done.
	printf("\nRead Thread is closing the driver...\n");
	CloseHandle(drv_rx);
	printf("Read Thread is terminating...\n");
    return 0; 
}
 
int PrintRxData(PSTREAMING_READ_INFO pri)
{
	USHORT datlen, ln;

	if(!pri->MiscBits)
	{	//a frame was received - dump it without first 16 header bytes
		datlen = pri->FrameLength - NUMBER_OF_CRC_BYTES;

		printf("Rx datlen : %d. Data :\n", datlen);
		//Print it out without the two CRC bytes.
		for(ln = 0; ln < datlen; ln++)
		{	printf("%02X ", pri->Data[ln]);
			//do not print more than 100 bytes
			if(ln == 99)
			{	printf("...\n");
				break;
			}
		}

		good_rx_frames_count++;
		good_rx_bytes_count += datlen;
		return 0;
	}else
	{	//CRC err or Aborted frame.
		printf("MBits = 0x%02X\n", pri->MiscBits);
		bad_rx_frames_count++;
		return 1;
	}
}


// Write Thread
DWORD 
TxThreadFunc(
	LPDWORD lpdwParam
	)
{ 
	HANDLE	drv_Tx;
	DWORD dwWaitResult;

	tx_test_byte = 0;
	
    printf("\nTxThreadFunc:  thread parameter=%d\n", *lpdwParam); 
	
	//open another handle strictly for writing
	drv_Tx = CreateFile(
		device_name, 
		GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		(LPSECURITY_ATTRIBUTES)NULL,
		OPEN_EXISTING,
		FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH,
		(HANDLE)NULL
		);
	
    if (drv_Tx == INVALID_HANDLE_VALUE)
	{	printf("\nUnable to open %s for writing!\n", device_name);
		return(0);
	}
	
	//start the transmission
	while(c != 'q' && stop_tx == FALSE)
	{	
		//wait for tx event to be set to signaled state.
		dwWaitResult = WaitForSingleObject(hTxWaitHandle, 1000);
		
		switch(dwWaitResult)
		{
		case WAIT_TIMEOUT:
			continue;
		case WAIT_ABANDONED:
			printf("Tx WAIT_ABANDONED\n.");
			stop_tx = TRUE;
			break;
		}

		if(stop_tx == TRUE)
			break;

		//initialize tx_data structure
		tx_data.frame_length = TX_LENGTH;
		tx_data.operation_status = TX_TIMEOUT;
		memset(tx_data.frame_data, tx_test_byte, TX_LENGTH);
		
		// queue for transmission
		if(	DoWriteCommand(drv_Tx, &tx_data))
		{	printf("Communications disabled or previouse DoWriteCommand()\
has not completed yet!\n");
			break;
		}
		
		//check that frame was transmitted
		switch(tx_data.operation_status)
		{
		case TX_OK:
			tx_bytes_count += TX_LENGTH;
			tx_frames_count++;

			//This will generate values from 0x00 to 0xFF.
			//Advance to the next value.
			tx_test_byte++;
			printf("Frame queued for transmission.\n");
			break;
					
		case TX_TIMEOUT:
			//it means that tx buffers are constanly full,
			//usually because the line is disconnected
			printf("****** TX_TIMEOUT ******\n");
			break;
					
		case TX_RETRY:
			//a frame was not transmitted !! should be retransmitted.
			printf("Retry Tx!\n");
			break;
					
		case TX_DATA_TOO_LONG:
			//maximum length is:
			//PRI_MAX_NO_DATA_BYTES_IN_FRAME bytes for Primary   port,
			//SEC_MAX_NO_DATA_BYTES_IN_FRAME bytes for Secondary port.
			printf("****** TX_DATA_TOO_LONG ******\n");
			break;
					
		case TX_DATA_TOO_SHORT:
			//minimum is 1 byte  for Primary   port,
			//			 2 bytes for Secondary port
			printf("****** TX_DATA_TOO_SHORT ******\n");
			break;

		default:
			printf("Unknown return code (0x%X) on transmission!\n",
				tx_data.operation_status);
			stop_tx = FALSE;
			break;
		}
	}//while()

	CloseHandle(hTxWaitHandle);

	printf("\nWrite Thread is closing the driver...\n");
	CloseHandle(drv_Tx);
	printf("Write Thread is terminating...\n");
    return 0; 
}

void print_stats(HANDLE drv)
{
	COMMS_ERROR_STATS_STRUCT * comms_err_stats = (COMMS_ERROR_STATS_STRUCT *)pcb->data;
	GLOBAL_STATS_STRUCT * glob_stats = (GLOBAL_STATS_STRUCT *)pcb->data;

	printf("\n\n");
	printf("good_rx_frames_count: %ld\n", good_rx_frames_count);
	printf("good_rx_bytes_count : %ld\n", good_rx_bytes_count);
	
	printf("bad_rx_frames_count : %ld\n", bad_rx_frames_count);
	
	printf("tx_bytes_count  : %u\n", tx_bytes_count);
	printf("tx_frames_count : %u\n", tx_frames_count);
	
	printf("\n\n");
	pcb->command = READ_COMMS_ERROR_STATS;
	pcb->buffer_length = 0;
	printf("Performing READ_COMMS_ERROR_STATS\n");
	DoCommand(drv, pcb);

	if (pcb->return_code == 0) 
	{	printf("Rx_overrun_err_count %u\n", comms_err_stats->Rx_overrun_err_count);
		printf("CRC_err_count        %u\n", comms_err_stats->CRC_err_count);
		printf("Rx_abort_count       %u\n", comms_err_stats->Rx_abort_count);
		printf("Rx_dis_pri_bfrs_full_count %u\n", 
			comms_err_stats->Rx_dis_pri_bfrs_full_count);

		printf("missed_Tx_und_int_count %u\n", comms_err_stats->missed_Tx_und_int_count);
		printf("DCD_state_change_count  %u\n", comms_err_stats->DCD_state_change_count);
		printf("CTS_state_change_count  %u\n", comms_err_stats->CTS_state_change_count);
	}else
	{	printf("READ_COMMS_ERROR_STATS failed! rc 0x%X\n", pcb->return_code);
	}

	printf("\n\n");
	pcb->command = READ_GLOBAL_STATISTICS;
	pcb->buffer_length = 0;
	printf("Performing READ_GLOBAL_STATISTICS\n");
	DoCommand(drv, pcb);
	
	if (pcb->return_code == 0)
	{	printf("app_IRQ_timeout_count : %d", glob_stats->app_IRQ_timeout_count);
	}else
	{	printf("READ_GLOBAL_STATISTICS failed! rc 0x%X\n", pcb->return_code);
	}
	printf("\n");
}


void get_device_stats(HANDLE drv)
{
	DEVICE_STATISTICS dev_stats;

	printf("\n***** get_device_stats() *****\n");

	dev_stats.return_code = 0xaa;

	ReadDeviceStatistics(drv, &dev_stats);

	if(dev_stats.return_code == 0)
	{
		printf("number_of_frames_received_no_err: %u\n",
			dev_stats.number_of_frames_received_no_err);

		printf("number_of_frames_discarded_rx_q_full: %u\n",
			dev_stats.number_of_frames_discarded_rx_q_full);

		printf("number_of_frames_excessive_length: %d\n",
			dev_stats.number_of_frames_excessive_length);

		printf("number_of_too_short_frames: %u\n",
			dev_stats.number_of_too_short_frames);

		printf("number_of_times_rx_buffers_out_of_synch_with_card: %u\n",
			dev_stats.number_of_times_rx_buffers_out_of_synch_with_card);

		printf("\nget_device_stats() - OK\n");

	}else
	{	printf("\nget_device_stats() - failed!! return_code : 0x%X\n",
			dev_stats.return_code);
	}
	printf("\n");
}


int set_device_configuration(HANDLE drv)
{
	DEVICE_CONFIGURATION dev_cfg;

	printf("\n***** set_device_configuration() *****\n");

	dev_cfg.return_code = 0xaa;
	dev_cfg.maximum_length_of_rx_queue = 500;	
	dev_cfg.maximum_rx_data_length = 2048;

	SetDeviceConfiguration(drv, &dev_cfg);
	
	if(dev_cfg.return_code == DEVICE_CFG_OK)
	{
		printf("\nset_device_configuration(): DEVICE_CFG_OK\n");
		return 0;
	}else
	{	printf("\nset_device_configuration(): failed!! : 0x%X\n", dev_cfg.return_code);
		return 1;
	}
}
