/*
	Sangstrm.H - Streaming Device Driver (S508/S514) User Interface Header
	Copyright (c) 2001 Sangoma Technologies Inc.
	v 5,0,0,0 - June 9, 2004

	Set tabs to 4 spaces when viewing.
*/

// For Version 5,0,0,0 and later of the API driver

//
// DeviceIoControl command codes. Must include "winioctl.h" in app for
// definition of CTL_CODE and defines.
//

////////////////////////////////////////////////////////////////////////////////////
// Standard CBLOCK-passing command:
// Input: ptr to CHDLC_MAILBOX_STRUCT structure
// Output: ptr to CHDLC_MAILBOX_STRUCT structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
#define IoctlCodeCblockCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 10, METHOD_BUFFERED, FILE_ANY_ACCESS)

////////////////////////////////////////////////////////////////////////////////////
// Command to send a frame:
// Input : ptr to TX_DATA_STRUCT structure
// Output: ptr to TX_DATA_STRUCT structure (may be same as input)
// Uses Direct I/O (Irp->MdlAddress). Synchronous call.
// Note: Frame only queued for transmission when 'operation_status' is TX_OK.
//		 If not, must repeat this command with reinitialized structure.
#define IoctlCodeWriteCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 11, METHOD_OUT_DIRECT, FILE_ANY_ACCESS)

//
//Structure used for transmission
//
typedef struct _TX_DATA_STRUCT{

	unsigned char  operation_status;
	unsigned short frame_length;
	unsigned char  frame_data[PRI_MAX_NO_DATA_BYTES_IN_FRAME];

}TX_DATA_STRUCT;

//possible values for 'operation_status' on return
#define TX_OK				0
#define TX_TIMEOUT			1
#define TX_RETRY			3
#define TX_DATA_TOO_LONG	4
#define TX_DATA_TOO_SHORT	5

////////////////////////////////////////////////////////////////////////////////////
// Command to read a frame:
// Input: NULL
// Output: ptr to STREAMING_READ_INFO (plus adequate data storage area)
// Uses Direct I/O (Irp->MdlAddress). Blocking call on device handle.
// Note: Since this command blocks all further I/O requests, it must be done
//       on a separate device handle. Just open the same device again and use
//       this handle for reading.
#define IoctlCodeReadCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 12, METHOD_IN_DIRECT, FILE_ANY_ACCESS)

//
// Equivalent to Receive Status Element in Manual. Must be packed on 1-byte boundary.
// Data of variable length follows.
//
#pragma pack(1)

typedef struct _STREAMING_READ_INFO {
	UCHAR Status;					// set to 0x01 on return if frame read
	USHORT FrameLength;				// set to length of frame that follows
	UCHAR MiscBits;					// various frame-related bits
	USHORT TimeStamp;				// timestamp of frame in milliseconds
	UCHAR Reserved[10];
	UCHAR Data[1];					// variable-length frame data
} STREAMING_READ_INFO, *PSTREAMING_READ_INFO;

#pragma pack()

//
// Possible values for Status member of STREAMING_READ_INFO structure
// when IoctlCodeReadCommand returns.
//
#define RX_DATA_AVAILABLE	0x01	//Data was read.
#define RX_DATA_TIMEOUT		0x03	//Notification that no data was received in the
									//last 100 milliseconds.

////////////////////////////////////////////////////////////////////////////////////
// Command to get devices available for an API caller.
// This command is used only by Configurator.exe.
// Input: NULL
// Output: ptr to SANGSTRM_DEVICE structure
// Uses Buffered I/O. Synchronous call. Used by
// configurator program only.
#define IoctlGetDevicesCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 13, METHOD_BUFFERED, FILE_ANY_ACCESS)

#pragma pack(1)
//
//Structre used by IoctlGetDevicesCommand
//
typedef struct{
	USHORT HWareType;//CardType;
	
	ULONG  BusNumber;
	ULONG  SlotNumber;
	USHORT CPU;
	USHORT comms_port;
	
	ULONG  Memory;
	ULONG  IRQ;
	ULONG  IO_Port;

}SANGSTRM_DEVICE;

#pragma pack()

////////////////////////////////////////////////////////////////////////////////////
// Command to set T1/E1 configuration on S5144/S5147 cards.
// The default configuration (set on driver start up) is:
//
// media	= WANOPT_MEDIA_T1;
// lcode	= WANOPT_LC_B8ZS;
// frame	= WANOPT_FR_ESF;
// lbo		= WANOPT_T1_LBO_0_DB;
// te_clock	= WANOPT_NORMAL_CLK;
// active_ch= TE_ENABLE_ALL_CHANNELS;
//
// Input: ptr to TE_CFG_T structure
// Output: ptr to TE_CFG_T structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
// On return check the return_code member.
#define IoctlSetTEConfiguration CTL_CODE(FILE_DEVICE_UNKNOWN, 14, METHOD_BUFFERED, FILE_ANY_ACCESS)

////////////////////////////////////////////////////////////////////////////////////
// Command to get T1/E1/56K alarms, activate/deactivate line test modes.
// Input: ptr to CHDLC_MAILBOX_STRUCT structure.
// Output: ptr to CHDLC_MAILBOX_STRUCT structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
#define Ioctl_TE_56K_MonitorCommand CTL_CODE(FILE_DEVICE_UNKNOWN, 15, METHOD_BUFFERED, FILE_ANY_ACCESS)

////////////////////////////////////////////////////////////////////////////////////
// Internal use only.
// Undocumented.
#define Ioctl_GET_MEM_DUMP CTL_CODE(FILE_DEVICE_UNKNOWN, 16, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
//Memory dump definitions
//
#define HEX_BASE				16				// 
#define MEMORY_DEF_ADDR			"00000"			// default start memory addr
#define MEMORY_DEF_LEN			"100"			// default dump memory length
#define MAX_SIZEOF_S508_MEMORY	(128*1024)

//Memory Dump sturcture
typedef struct 
{
	UCHAR	return_code;
	ULONG	offset;
	USHORT	length;
	UCHAR	data[2000];
} MEMORY_DUMP, *PMEMORY_DUMP;

//
//End of Memory dump definitions.
//

////////////////////////////////////////////////////////////////////////////////////
// Command to get per- API device (i.g. SANGSTRM0) statistics.
// Input: ptr to DEVICE_STATISTICS structure
// Output: ptr to DEVICE_STATISTICS structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
#define IoctlReadDeviceStatistics CTL_CODE(FILE_DEVICE_UNKNOWN, 17, METHOD_BUFFERED, FILE_ANY_ACCESS)

typedef struct _DEVICE_STATISTICS
{
	UCHAR return_code;
	ULONG number_of_frames_received_no_err;
	ULONG number_of_frames_discarded_rx_q_full;

	ULONG number_of_frames_excessive_length;
	ULONG number_of_too_short_frames;

	ULONG number_of_times_rx_buffers_out_of_synch_with_card;

}DEVICE_STATISTICS, *PDEVICE_STATISTICS;

////////////////////////////////////////////////////////////////////////////////////
// Command to set per- API device (i.g. SANGSTRM0) configuration.
//
// The default configuration (set on driver start up) is:
//
//	maximum_length_of_rx_queue = 500
//	maximum_rx_data_length     = 2048
//
// Should be used only if COMMUNICATIONS disabled.
// Input: ptr to DEVICE_CONFIGURATION structure
// Output: ptr to DEVICE_CONFIGURATION structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
#define IoctlSetDeviceConfiguration CTL_CODE(FILE_DEVICE_UNKNOWN, 18, METHOD_BUFFERED, FILE_ANY_ACCESS)

typedef struct _DEVICE_CONFIGURATION
{
	UCHAR return_code;

	ULONG maximum_length_of_rx_queue;
	ULONG maximum_rx_data_length;		//including CRC bytes

}DEVICE_CONFIGURATION, *PDEVICE_CONFIGURATION;

#define MINIMUM_LENGTH_OF_RX_QUEUE	500
#define MAXIMUM_LENGTH_OF_RX_QUEUE	10000
#define MINIMUM_LENGTH_OF_RX_DATA	2

//on return the "return_code" member will be set to one of the following:
enum DEVICE_CONFIGURATION_RETURN_CODE{

	DEVICE_CFG_OK = 1,
	DEVICE_CFG_FAILED_COMMS_ENABLED,
	DEVICE_CFG_FAILED_INVALID_LENGTH_OF_RX_QUEUE,	//must be between MINIMUM_LENGTH_OF_RX_QUEUE
													//and MAXIMUM_LENGTH_OF_RX_QUEUE

	DEVICE_CFG_FAILED_INVALID_RX_DATA_LENGTH,		//must be at least	MINIMUM_LENGTH_OF_RX_DATA
	DEVICE_CFG_FAILED_MEMORY_ALLOCATION_ERR
};

////////////////////////////////////////////////////////////////////////////////////

// return codes from IoctlSetTEConfiguration commands
#define COMMAND_PERFORMED_SUCCESSFULLY		0x00		

// SET_CHDLC_CONFIGURATION must be done	before using this command
#define DO_SET_CHDLC_CONFIGURATION      	0x01 

// DISABLE_CHDLC_COMMUNICATIONS must be done before using this command
#define DO_DISABLE_CHDLC_COMMUNICATIONS		0x02 
									
/* Failed to set T1/E1 configuration. 
   Check values passed to IoctlSetTEConfiguration command.
   Make sure the card type is S5144 or S5147. 
   Check messages in the driver log file. 
   The log file is /WINNT/System32/drivers/strmlog.txt.
*/
#define TE_CONFIGURATION_FAILED				0x03 

//
// Driver-defined return codes for CBLOCK-associated commands.
//
#define RCODE_TIMEOUT_ERROR		0x7f	// Card is not responding to command.
#define RCODE_INIT_ERROR		0x7e	// Card could not be initialized.
										// (SET_CHDLC_CONFIGURATION only)
#define NUMBER_OF_CRC_BYTES		2


//
//T1/E1 related declarations
//
//
// Possible values for members of TE_CFG_T structure:
//
/* Possible values for 'media' member */
/* The media (connection) type */
#define WANOPT_MEDIA_NONE		0x00	/* no DSU/CSU or old T1 connection */
#define WANOPT_MEDIA_T1			0x01	/* T1 connection */
#define WANOPT_MEDIA_E1			0x02	/* E1 connection */
#define WANOPT_MEDIA_56K		0x03	/* 56K connection */

/* End of values for 'media' member */
/*==================================*/

/* Possible values for 'lcode' member */
/* The line code */
#define WANOPT_LC_AMI           0x01	/* for T1 and E1 */
#define WANOPT_LC_B8ZS          0x02	/* for T1 only */
#define WANOPT_LC_HDB3          0x03	/* for E1 only */
/* End of values for 'lcode' member */
/*==================================*/

/* Possible values for 'frame' member */
/* The Frame mode (T1) */
#define WANOPT_FR_ESF           0x01
#define WANOPT_FR_D4            0x02
#define WANOPT_FR_ESF_JAPAN     0x03

/* The frame mode (E1) */
#define WANOPT_FR_CRC4          0x04
#define WANOPT_FR_NCRC4         0x05
/* End of values for 'frame' member */
/*==================================*/

/* Possible values for 'lbo' member */
/* Line Build Out */
/* For T1 only */
#define WANOPT_T1_LBO_0_DB      0x01
#define WANOPT_T1_LBO_75_DB     0x02
#define WANOPT_T1_LBO_15_DB     0x03
#define WANOPT_T1_LBO_225_DB    0x04
#define WANOPT_T1_0_110         0x05
#define WANOPT_T1_110_220       0x06
#define WANOPT_T1_220_330       0x07
#define WANOPT_T1_330_440       0x08
#define WANOPT_T1_440_550       0x09
#define WANOPT_T1_550_660       0x0A

/* For E1 only */
#define WANOPT_E1_120           0x0B
#define WANOPT_E1_75            0x0C
/* End of values for 'lbo' member */
/*==================================*/

/* Possible values for 'te_clock' member */
/* Clocking Master/Normal */
#define WANOPT_NORMAL_CLK	0x01
#define WANOPT_MASTER_CLK	0x02
/* End of values for 'te_clock' member */
/*=====================================*/

/* Possible values for 'active_ch' member */
/* Active Channels (active_ch) */
#define TE_ENABLE_ALL_CHANNELS	0x7FFFFFFF

/* To enable selected channels set bits in 'active_ch'.
   For example: if bit 0 is set, channel 1 is enabled,
   if bit 0 is reset, channel 1 is disabled.*/
/* End of values for 'active_ch' member */
/*=====================================*/

typedef struct TE_CFG {
	unsigned char media;
	unsigned char lcode;
	unsigned char frame;
	unsigned char lbo;
	unsigned char te_clock;
	unsigned long active_ch;
	unsigned char return_code;
} TE_CFG_T;

//Test modes for TE1 cards.
#define ACTIVATE_LINELB_MODE	0xA0
#define DEACTIVATE_LINELB_MODE	0xA1
#define ACTIVATE_PAYLB_MODE		0xA2
#define DEACTIVATE_PAYLB_MODE	0xA3
#define ACTIVATE_DDLB_MODE		0xA4
#define DEACTIVATE_DDLB_MODE	0xA5
#define ACTIVATE_LB_MODE		0xA6
#define DEACTIVATE_LB_MODE		0xA7

//command to get T1/E1/56K alarms
#define TE1_56K_STAT			0x62
//command to read a PMC register
#define READ_PMC_REGISTER		0x65

////////////////////////////////////////////////////////////////////////////////////

